from typing import Any, Dict, Type, TypeVar, Union, List

import attr

from ..models.calculation_composition import CalculationComposition
from ..models.api_value_component_composition import ApiValueComponentComposition
from ..types import UNSET, Unset

T = TypeVar("T", bound="ApiOutputCalculationResultPhaseComposition")


@attr.s(auto_attribs=True)
class ApiOutputCalculationResultPhaseComposition:
    """Holds composition information for a phase.
    
    Attributes
    ----------
    composition_units : str
        Units of the composition.
    molar_mass_units : str
        Units of the molar mass.
    components : List[ApiValueComponentComposition]
        Composition of the phase.
    """
    composition_units: Union[Unset, None, str] = UNSET
    molar_mass_units: Union[Unset, None, str] = UNSET
    components: Union[Unset, List[ApiValueComponentComposition]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Dump `ApiOutputCalculationResultPhaseComposition` instance to a dict."""
        composition_units = self.composition_units
        molar_mass_units = self.molar_mass_units
        components: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.components, Unset):
            components = []
            for components_item_data in self.components:
                components.append(components_item_data.to_dict())

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if not isinstance(composition_units, Unset):
            field_dict["compositionUnits"] = composition_units
        if not isinstance(molar_mass_units, Unset):
            field_dict["molarMassUnits"] = molar_mass_units
        if not isinstance(components, Unset):
            field_dict["components"] = components

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Load `ApiOutputCalculationResultPhaseComposition` instance from a dict."""
        d = src_dict.copy()
        composition_units: str = d.pop("compositionUnits", UNSET)

        molar_mass_units: str = d.pop("molarMassUnits", UNSET)

        _components: List[Dict[str, Any]] = d.pop("components", UNSET)
        components: Union[Unset, List[ApiValueComponentComposition]]
        if isinstance(_components, Unset):
            components = UNSET
        else:
            components = []
            for components_item_data in _components:
                components.append(ApiValueComponentComposition.from_dict(components_item_data))

        api_output_calculation_result_phase_composition = cls(
            composition_units=composition_units,
            molar_mass_units=molar_mass_units,
            components=components,
        )

        return api_output_calculation_result_phase_composition

    def to_calculation_composition(self) -> List[CalculationComposition]:
        """
        Convert `ApiOutputCalculationResultPhaseComposition` to `List[CalculationComposition]`.
        
        Returns
        -------
        composition : List[CalculationComposition]
            Composition of the phase in format suitable for input to classes like `FlashFixedTemperaturePressureCalculationInput.components`.
        """
        composition: List[CalculationComposition] = []
        for component in self.components:
            composition.append(
                CalculationComposition(
                    component_name=component.name,
                    amount=component.value,
                )
            )
        return composition

    def __str__(self) -> str:
        """
        Returns a string representation of the phase composition.
        """
        units = self.composition_units if not isinstance(self.composition_units, Unset) else "N/A"
        mm_units = self.molar_mass_units if not isinstance(self.molar_mass_units, Unset) else "N/A"
        lines = [f"Units: {units}, Molar Mass Units: {mm_units}"]

        if not isinstance(self.components, Unset) and self.components is not None:
            lines.append(str(self.components))
        else:
            lines.append("Composition: N/A")

        return "\n".join(lines)



import json
from pathlib import Path
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.calculation_composition import CalculationComposition
from ..models.api_output_calculation_result_phase_composition import ApiOutputCalculationResultPhaseComposition
from ..models.api_output_calculation_result_phase import ApiOutputCalculationResultPhase
from ..models.api_value_pressure import ApiValuePressure
from ..models.api_value_temperature import ApiValueTemperature
from ..types import UNSET, Unset

T = TypeVar("T", bound="ApiOutputCalculationResultPoint")


@attr.s(auto_attribs=True)
class ApiOutputCalculationResultPoint:
    """Result for a point.
    
    Attributes
    ----------
    temperature : ApiValueTemperature
        Temperature of the system.
    pressure : ApiValuePressure
        Pressure of the system.
    phases : List[ApiOutputCalculationResultPhase]
        List of phases in the system. The first index is the system (or input) phase, and the rest are the output phases.
    
    Notes
    -----
    1. By default, the output phases are ordered by density descending.
    """
    temperature: Union[Unset, ApiValueTemperature] = UNSET
    pressure: Union[Unset, ApiValuePressure] = UNSET
    phases: Union[Unset, None, List[ApiOutputCalculationResultPhase]] = UNSET

    # region: Utility methods
    def sort_output_phases(self, sort_key: str, ascending: bool = False, inplace=True) -> None:
        """Order the output phases according to the `sort_key`. The output phases are in `phases[1:]` and are re-ordered in-place.
        
        Parameters
        ----------
        sort_key : str
            The key to sort the phases by. 
            Options are: `{'volume', 'density', 'molecular_weight', 'enthalpy', 'entropy', 'cp', 'cv', 'solubility_parameter', 'compressibility', 'mole_percent', 'weight_percent', 'jt_coefficient', 'speed_of_sound'}`.
        ascending : bool, optional
            Whether to sort the phases in ascending order.
        """
        allowed_keys = {
            'volume', 'density', 'molecular_weight', 'enthalpy', 'entropy',
            'cp', 'cv', 'solubility_parameter', 'compressibility',
            'mole_percent', 'weight_percent', 'jt_coefficient', 'speed_of_sound'
        }

        if sort_key not in allowed_keys:
            raise ValueError(f"Unsupported sort_key: {sort_key!r}")

        if not isinstance(self.phases, list) or len(self.phases) < 2:
            return  # Nothing to sort
        
        # Extract the phases (only the output phases are re-ordered)
        system_phase = self.phases[0]
        output_phases = self.phases[1:]

        def get_sort_value(phase):
            attr = getattr(phase, sort_key, None)
            return getattr(attr, "value", float("inf")) if attr is not None else float("inf")

        output_phases.sort(key=get_sort_value, reverse=not ascending)
        phases = [system_phase] + output_phases
        if inplace:
            self.phases = phases
        return phases

    def phases_to_calculation_composition(self, 
        sort_key='molecular_weight', 
        ascending=False
    ) -> Dict[str, List[CalculationComposition]]:
        """Convert the output compositions (`ApiOutputCalculationResultPhaseComposition`) to input compositions type (`List[CalculationComposition]`).

        Parameters
        ----------
        sort_key : str, optional
            The key to sort the phases by. defaults to 'molecular_weight'.
            Options are: `{'volume', 'density', 'molecular_weight', 'enthalpy', 'entropy', 'cp', 'cv', 'solubility_parameter', 'compressibility', 'mole_percent', 'weight_percent', 'jt_coefficient', 'speed_of_sound'}`.
        ascending : bool, optional
            Whether to sort the phases in ascending order. Defaults to False.
        
        Returns
        -------
        compositions : Dict[str, List[CalculationComposition]]
            The keys are phase labels and the values are compositions.
        """
        compositions = {}
        phases = self.sort_output_phases(sort_key=sort_key, ascending=ascending, inplace=False)
        for phase in phases:
            phase_composition: ApiOutputCalculationResultPhaseComposition = phase.composition
            compositions[phase.phase_label] = phase_composition.to_calculation_composition()
        return compositions
    # endregion
    
    # region: Serialization methods
    def to_dict(self) -> Dict[str, Any]:
        """Dump `ApiOutputCalculationResultPoint` instance to a dict."""
        temperature: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.temperature, Unset):
            temperature = self.temperature.to_dict()

        pressure: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.pressure, Unset):
            pressure = self.pressure.to_dict()

        phases: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.phases, Unset):
            if self.phases is None:
                phases = None
            else:
                phases = []
                for phases_item_data in self.phases:
                    phases_item = phases_item_data.to_dict()
                    phases.append(phases_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if not isinstance(temperature, Unset):
            field_dict["temperature"] = temperature
        if not isinstance(pressure, Unset):
            field_dict["pressure"] = pressure
        if not isinstance(phases, Unset):
            field_dict["phases"] = phases

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Load `ApiOutputCalculationResultPoint` instance from a dict."""
        d = src_dict.copy()
        _temperature = d.pop("temperature", UNSET)
        temperature: Union[Unset, ApiValueTemperature]
        if isinstance(_temperature, Unset):
            temperature = UNSET
        else:
            temperature = ApiValueTemperature.from_dict(_temperature)

        _pressure = d.pop("pressure", UNSET)
        pressure: Union[Unset, ApiValuePressure]
        if isinstance(_pressure, Unset):
            pressure = UNSET
        else:
            pressure = ApiValuePressure.from_dict(_pressure)

        phases = []
        _phases = d.pop("phases", UNSET)
        for phases_item_data in _phases or []:
            phases_item = ApiOutputCalculationResultPhase.from_dict(phases_item_data)

            phases.append(phases_item)

        api_output_calculation_result_point = cls(
            temperature=temperature,
            pressure=pressure,
            phases=phases,
        )

        return api_output_calculation_result_point

    def to_json(self, path: Path = None) -> Union[str, None]:
        """Dump `ApiOutputCalculationResultPoint` instance to a JSON string or file.
        
        Parameters
        ----------
        path : Path, optional
            If provided, the JSON string will be written to this path.
            Else the JSON string will be returned.
        """
        if path is None:
            return json.dumps(self.to_dict())
        else:
            try:
                path = Path(path)
                path.parent.mkdir(parents=True, exist_ok=True)
                with open(path, 'w') as f:
                    json.dump(self.to_dict(), f)
            except Exception as e:
                print(e)
    
    @classmethod
    def from_json(cls: Type[T], path: Path) -> T:
        """Load `ApiOutputCalculationResultPoint` instance from a JSON file.
        
        Parameters
        ----------
        path : Path
            Path to the JSON file.
        """
        with open(path, 'r') as f:
            data = json.load(f)
        return cls.from_dict(data)

    # region: Magic methods
    def __str__(self, include_compositions: bool=True) -> str:
        """String representation of the flash calculation result."""
        def format_num(x):
            try:
                if isinstance(x, (int, float)):
                    return f"{float(x):.4f}"
            except Exception:
                pass
            return str(x)

        # Build header: first two columns are fixed ("Property" and "Unit")
        # Then one column per phase (using phase_label as header)
        phase_labels = [phase.phase_label for phase in self.phases]
        headers = ["Property", "Unit"] + phase_labels

        rows = []
        phase_count = len(self.phases)

        # Add Temperature row: value only in the first (System) phase column
        temp_obj = self.temperature
        temp_val = getattr(temp_obj, "value", None)
        temp_unit = getattr(temp_obj, "units", "")
        temp_formatted = format_num(temp_val) if temp_val is not None else "N/A"
        temp_row = ["Temperature", temp_unit] + [temp_formatted] + [""] * (phase_count - 1)
        rows.append(temp_row)

        # Add Pressure row: value only in the first (System) phase column
        press_obj = self.pressure
        press_val = getattr(press_obj, "value", None)
        press_unit = getattr(press_obj, "units", "")
        press_formatted = format_num(press_val) if press_val is not None else "N/A"
        press_row = ["Pressure", press_unit] + [press_formatted] + [""] * (phase_count - 1)
        rows.append(press_row)

        # List of phase-level properties to display (rows below Temperature/Pressure)
        phase_props = [
            ("Volume", "volume"),
            ("Density", "density"),
            ("Entropy", "entropy"),
            ("Enthalpy", "enthalpy"),
            ("Cp", "cp"),
            ("Cv", "cv"),
            ("JT Coefficient", "jt_coefficient"),
            ("Speed of Sound", "speed_of_sound"),
            ("Solubility Parameter", "solubility_parameter"),
            ("Molecular Weight", "molecular_weight"),
            ("Compressibility", "compressibility"),
            ("Mole Percent", "mole_percent"),
            ("Weight Percent", "weight_percent"),
        ]

        # For each property, build a row with property name, unit, and the value for each phase
        for disp_name, attr_name in phase_props:
            # Determine the unit from the first phase that has this attribute
            unit = ""
            for phase in self.phases:
                attr_obj = getattr(phase, attr_name, None)
                if attr_obj is not None:
                    unit = getattr(attr_obj, "units", "")
                    break
            row = [disp_name, unit]
            for phase in self.phases:
                attr_obj = getattr(phase, attr_name, None)
                if attr_obj is None:
                    row.append("N/A")
                else:
                    value = getattr(attr_obj, "value", attr_obj)
                    row.append(format_num(value))
            rows.append(row)

        # If compositions should be included...
        if include_compositions:
            # Add a divider row
            divider_row = ["-" * 5] * len(headers)
            rows.append(divider_row)
            # Check if the first phase has a composition
            if self.phases and hasattr(self.phases[0], "composition") and self.phases[0].composition is not None:
                comp0 = self.phases[0].composition
                comp_units = getattr(comp0, "composition_units", "")
                components = comp0.components
                # For each composition component, add a row
                for i, comp in enumerate(components):
                    # Place composition_units only in the first composition row
                    unit_str = comp_units if i == 0 else ""
                    prop_name = comp.name
                    row = [prop_name, unit_str]
                    # For each phase, get the corresponding composition value (assume same ordering)
                    for phase in self.phases:
                        if hasattr(phase, "composition") and phase.composition is not None:
                            try:
                                comp_arr = phase.composition.components
                                phase_comp = comp_arr[i]
                                val = getattr(phase_comp, "value", "N/A")
                                row.append(format_num(val))
                            except IndexError:
                                row.append("N/A")
                        else:
                            row.append("N/A")
                    rows.append(row)
                
        # Determine column widths
        all_rows = [headers] + rows
        num_cols = len(headers)
        col_widths = [
            max(len(str(all_rows[r][c])) for r in range(len(all_rows)))
            for c in range(num_cols)
        ]

        # Format the header and rows
        header_line = " | ".join(str(item).ljust(width) for item, width in zip(headers, col_widths))
        separator = "-+-".join("-" * width for width in col_widths)
        row_lines = [
            " | ".join(str(item).ljust(width) for item, width in zip(row, col_widths))
            for row in rows
        ]

        # Build the table
        table = "\n".join([header_line, separator] + row_lines)
        title = "Flash Calculation Result Point"
        return "\n" + title + "\n\n" + table
    # endregion



""" `units`

This module defines the `Units` class, which represents unit assignments for properties in flash calculations.
"""
import attr
from typing import Dict

@attr.s(auto_attribs=True)
class Units:
    """
    Represents unit assignments for properties in flash calculations.
    Users can pick a preset (SI, US, EU) or override individual units.
    
    Attributes
    ----------
    C_in : str
        Unit for composition (input). Options: `{"Massfraction", "Molefraction", "Masspercent", "Molepercent"}`.
    C_out : str
        Unit for composition (output). Options: `{"Massfraction", "Molefraction", "Masspercent", "Molepercent"}`.
    T_in : str
        Unit for temperature (input). Options: `{"Kelvin", "Celsius", "Fahrenheit", "Rankine"}`.
    T_out : str
        Unit for temperature (output). Options: `{"Kelvin", "Celsius", "Fahrenheit", "Rankine"}`.
    P_in : str
        Unit for pressure (input). Options: `{"Bar", "Atm", "Pa", "kPa", "MPa", "Psi", "Psig", "mmHg"}`.
    P_out : str
        Unit for pressure (output). Options: `{"Bar", "Atm", "Pa", "kPa", "MPa", "Psi", "Psig", "mmHg"}`.
    H_in : str
        Unit for enthalpy (input). Options: `{"kJ/Kg", "kJ/Ton", "kJ/kMol"}`.
    H_out : str
        Unit for enthalpy (output). Options: `{"kJ/Kg", "kJ/Ton", "kJ/kMol"}`.
    S_in : str
        Unit for entropy (input). Options: `{"kJ/(Kg Kelvin)", "kJ/(Ton Kelvin)", "kJ/(kMol Kelvin)"}`.
    S_out : str
        Unit for entropy (output). Options: `{"kJ/(Kg Kelvin)", "kJ/(Ton Kelvin)", "kJ/(kMol Kelvin)"}`.
    Cp_in : str
        Unit for heat capacity (input). Options: `{"kJ/(Kg Kelvin)", "kJ/(Ton Kelvin)", "kJ/(kMol Kelvin)"}`.
    Cp_out : str
        Unit for heat capacity (output). Options: `{"kJ/(Kg Kelvin)", "kJ/(Ton Kelvin)", "kJ/(kMol Kelvin)"}`.
    Viscosity_in : str
        Unit for viscosity (input). Options: `{"centiPoise"}`.
    Viscosity_out : str
        Unit for viscosity (output). Options: `{"centiPoise"}`.
    SurfaceTension_in : str
        Unit for surface tension (input). Options: `{"N/m"}`.
    SurfaceTension_out : str
        Unit for surface tension (output). Options: `{"N/m"}`.
    """
    C_in: str = attr.field(
        default="Massfraction",
        validator=attr.validators.in_(["Massfraction", "Molefraction", "Masspercent", "Molepercent"])
    )
    C_out: str = attr.field(
        default="Massfraction",
        validator=attr.validators.in_(["Massfraction", "Molefraction", "Masspercent", "Molepercent"])
    )
    T_in: str = attr.field(
        default="Kelvin",
        validator=attr.validators.in_(["Kelvin", "Celsius", "Fahrenheit", "Rankine"])
    )
    T_out: str = attr.field(
        default="Kelvin",
        validator=attr.validators.in_(["Kelvin", "Celsius", "Fahrenheit", "Rankine"])
    )
    P_in: str = attr.field(
        default="Bar",
        validator=attr.validators.in_(["Bar", "Atm", "Pa", "kPa", "MPa", "Psi", "Psig", "mmHg"])
    )
    P_out: str = attr.field(
        default="Bar",
        validator=attr.validators.in_(["Bar", "Atm", "Pa", "kPa", "MPa", "Psi", "Psig", "mmHg"])
    )
    H_in: str = attr.field(
        default="kJ/Kg",
        validator=attr.validators.in_(["kJ/Kg", "kJ/Ton", "kJ/kMol"])
    )
    H_out: str = attr.field(
        default="kJ/Kg",
        validator=attr.validators.in_(["kJ/Kg", "kJ/Ton", "kJ/kMol"])
    )
    S_in: str = attr.field(
        default="kJ/(Kg Kelvin)",
        validator=attr.validators.in_(["kJ/(Kg Kelvin)", "kJ/(Ton Kelvin)", "kJ/(kMol Kelvin)"])
    )
    S_out: str = attr.field(
        default="kJ/(Kg Kelvin)",
        validator=attr.validators.in_(["kJ/(Kg Kelvin)", "kJ/(Ton Kelvin)", "kJ/(kMol Kelvin)"])
    )
    Cp_in: str = attr.field(
        default="kJ/(Kg Kelvin)",
        validator=attr.validators.in_(["kJ/(Kg Kelvin)", "kJ/(Ton Kelvin)", "kJ/(kMol Kelvin)"])
    )
    Cp_out: str = attr.field(
        default="kJ/(Kg Kelvin)",
        validator=attr.validators.in_(["kJ/(Kg Kelvin)", "kJ/(Ton Kelvin)", "kJ/(kMol Kelvin)"])
    )
    Viscosity_in: str = attr.field(
        default="centiPoise",
        validator=attr.validators.in_(["centiPoise"])
    )
    Viscosity_out: str = attr.field(
        default="centiPoise",
        validator=attr.validators.in_(["centiPoise"])
    )
    Surfacetension_in: str = attr.field(
        default="N/m",
        validator=attr.validators.in_(["N/m"])
    )
    Surfacetension_out: str = attr.field(
        default="N/m",
        validator=attr.validators.in_(["N/m"])
    )

    @classmethod
    def SI(cls, **overrides: Dict[str, str]) -> "Units":
        """Returns `Units` with standard SI defaults, allowing overrides."""
        base = {
            "C_in": "Massfraction",     "C_out": "Massfraction",
            "T_in": "Kelvin",           "T_out": "Kelvin",
            "P_in": "Bar",              "P_out": "Bar",
            "H_in": "kJ/Kg",            "H_out": "kJ/Kg",
            "S_in": "kJ/(Kg Kelvin)",   "S_out": "kJ/(Kg Kelvin)",
            "Cp_in": "kJ/(Kg Kelvin)",  "Cp_out": "kJ/(Kg Kelvin)",
            "Viscosity_in": "centiPoise","Viscosity_out": "centiPoise",
            "Surfacetension_in": "N/m", "Surfacetension_out": "N/m",
        }
        base.update(overrides)
        return cls(**base)

    @classmethod
    def US(cls, **overrides: Dict[str, str]) -> "Units":
        """Returns US-style units: Fahrenheit & Psi, others same as SI."""
        return cls.SI(
            T_in="Fahrenheit",
            T_out="Fahrenheit",
            P_in="Psi",
            P_out="Psi",
            **overrides
        )

    @classmethod
    def EU(cls, **overrides: Dict[str, str]) -> "Units":
        """Returns European-style units: Celsius temps, others same as SI."""
        return cls.SI(
            T_in="Celsius",
            T_out="Celsius",
            **overrides
        )

    def __str__(self) -> str:
        """String representation: semicolon-separated unit assignments."""
        parts = [
            f"C(In,{self.C_in})", f"C(Out,{self.C_out})",
            f"T(In,{self.T_in})", f"T(Out,{self.T_out})",
            f"P(In,{self.P_in})", f"P(Out,{self.P_out})",
            f"H(In,{self.H_in})", f"H(Out,{self.H_out})",
            f"S(In,{self.S_in})", f"S(Out,{self.S_out})",
            f"Cp(In,{self.Cp_in})", f"Cp(Out,{self.Cp_out})",
            f"Viscosity(In,{self.Viscosity_in})", f"Viscosity(Out,{self.Viscosity_out})",
            f"Surfacetension(In,{self.Surfacetension_in})", f"Surfacetension(Out,{self.Surfacetension_out})",
        ]
        return ";".join(parts)

    def to_dict(self) -> Dict[str, str]:
        """Dump `Units` instance to a dict."""
        return {
            "C_in": self.C_in,
            "C_out": self.C_out,
            "T_in": self.T_in,
            "T_out": self.T_out,
            "P_in": self.P_in,
            "P_out": self.P_out,
            "H_in": self.H_in,
            "H_out": self.H_out,
            "S_in": self.S_in,
            "S_out": self.S_out,
            "Cp_in": self.Cp_in,
            "Cp_out": self.Cp_out,
            "Viscosity_in": self.Viscosity_in,
            "Viscosity_out": self.Viscosity_out,
            "Surfacetension_in": self.Surfacetension_in,
            "Surfacetension_out": self.Surfacetension_out,
        }

    @classmethod
    def from_dict(cls, data: Dict[str, str]) -> "Units":
        """Load `Units` instance from a dict."""
        return cls(**data)


from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.calculation_composition import CalculationComposition
from ..types import UNSET, Unset

T = TypeVar("T", bound="BatchFlashFixedTemperaturePressureCalculationItem")

@attr.s(auto_attribs=True)
class BatchFlashFixedTemperaturePressureCalculationItem:
    """Item for batch flash calculation.

    Attributes
    ----------
    temperature : float
        Temperature in units given in `BatchFlashFixedTemperaturePressureCalculationInput.units` attribute.
    pressure : float
        Pressure in units given in `BatchFlashFixedTemperaturePressureCalculationInput.units` attribute.
    """
    temperature: Union[Unset, float] = UNSET #Temperature in units given in 'Units' argument
    pressure: Union[Unset, float] = UNSET #Pressure in units given in 'Units' argument

    def to_dict(self) -> Dict[str, Any]:
        """Dump `BatchFlashFixedTemperaturePressureCalculationItem` instance to a dict."""

        temperature = self.temperature
        pressure = self.pressure

        field_dict: Dict[str, Any] = {}
        if temperature is not UNSET:
            field_dict["temperature"] = temperature
        if pressure is not UNSET:
            field_dict["pressure"] = pressure

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Load `BatchFlashFixedTemperaturePressureCalculationItem` instance from a dict."""
        d = src_dict.copy()

        temperature = d.pop("temperature", UNSET)
        pressure = d.pop("pressure", UNSET)

        flash_calculation_input = cls(
            temperature=temperature,
            pressure=pressure,
        )

        return flash_calculation_input



import pytest
from typing import Union
from equia.models import (
    CalculationComposition, ProblemDetails,
    BatchFlashFixedTemperaturePressureCalculationInput, BatchFlashCalculationResult, 
)
from equia.equia_client import EquiaClient
from equia.demofluids.demofluid1_nHexane_Ethylene_HDPE7 import demofluid1_nHexane_Ethylene_HDPE7
from equia.models.batchflash_fixed_temperature_pressure_calculation_item import BatchFlashFixedTemperaturePressureCalculationItem
from equia.units import Units
from utility.api_access import ApiAccess

@pytest.mark.asyncio
async def test_call_batchflash():
    client = EquiaClient(ApiAccess.url, ApiAccess.access_key)

    input: BatchFlashFixedTemperaturePressureCalculationInput = client.get_batchflash_fixed_temperature_pressure_input()
    input.fluid = demofluid1_nHexane_Ethylene_HDPE7()
    input.units = str(Units(C_in="Massfraction", C_out="Massfraction", T_in="Kelvin", T_out="Kelvin", P_in="Bar", P_out="Bar"))
    input.components = [
        CalculationComposition(amount=0.78),
        CalculationComposition(amount=0.02),
        CalculationComposition(amount=0.20)
    ]

    item1 = BatchFlashFixedTemperaturePressureCalculationItem()
    item1.temperature = 445
    item1.pressure = 20

    input.points.append(item1)

    result: Union[BatchFlashCalculationResult, ProblemDetails] = await client.call_batchflash_fixed_temperature_pressure_async(input)

    await client.cleanup()

    #assert result.status == 400
    assert result.success is True
    assert len(result.points[0].phases) == 4



from typing import Union
import pytest
from equia.models import (
    ProblemDetails,
    StatusInput, StatusResult
)
from equia.equia_client import EquiaClient
from utility.api_access import ApiAccess

@pytest.mark.asyncio
async def test_call_status():
    client = EquiaClient(ApiAccess.url, ApiAccess.access_key)

    input: StatusInput = client.get_status_input()

    result: Union[StatusResult, ProblemDetails] = await client.call_status_get_async(input)

    await client.cleanup()

    # Print the result for debugging purposes
    if isinstance(result, ProblemDetails):
        print("Error:", result)
    
    #assert result.status == 400
    assert result.success is True



import pytest
from typing import Union
from equia.models import (
    CalculationComposition, ProblemDetails,
    FlashFixedTemperaturePressureCalculationInput, FlashCalculationResult
)
from equia.equia_client import EquiaClient
from equia.demofluids.demofluid1_nHexane_Ethylene_HDPE7 import demofluid1_nHexane_Ethylene_HDPE7
from utility.api_access import ApiAccess

@pytest.mark.asyncio
async def test_call_flash():
    client = EquiaClient(ApiAccess.url, ApiAccess.access_key)

    input: FlashFixedTemperaturePressureCalculationInput = client.get_flash_fixed_temperature_pressure_input()
    input.temperature = 445
    input.pressure = 20
    input.components = [
        CalculationComposition(amount=0.78),
        CalculationComposition(amount=0.02),
        CalculationComposition(amount=0.20)
    ]
    input.flashtype = "FixedTemperaturePressure"

    input.fluid = demofluid1_nHexane_Ethylene_HDPE7()
    input.units = "C(In,Massfraction);C(Out,Massfraction);T(In,Kelvin);T(Out,Kelvin);P(In,Bar);P(Out,Bar);H(In,kJ/Kg);H(Out,kJ/Kg);S(In,kJ/(Kg Kelvin));S(Out,kJ/(Kg Kelvin));Cp(In,kJ/(Kg Kelvin));Cp(Out,kJ/(Kg Kelvin));Viscosity(In,centiPoise);Viscosity(Out,centiPoise);Surfacetension(In,N/m);Surfacetension(Out,N/m)"

    # Print the input for debugging purposes
    print(input)
    
    # Call the flash calculation asynchronously
    result: Union[FlashCalculationResult, ProblemDetails] = await client.call_flash_fixed_temperature_pressure_async(input)

    # Print the result for debugging purposes
    print(result)
    
    await client.cleanup()

    #assert result.status == 400
    assert result.success
    assert len(result.point.phases) == 4



""" `options`

Allowable specifications for flash calculations.
"""

from enum import Enum

class ToStrEnum(str, Enum):
    """Enum whose values are always strings, and which can validate+return .value."""
    @classmethod
    def to_str(cls, val: "ToStrEnum") -> str:
        # will raise if val isn’t a valid member
        return cls(val).value

class VolumeType(ToStrEnum):
    """Allowable volume calculation specifications: `{'Auto', 'Liquid', 'Vapor'}`."""
    Auto = 'Auto'
    Liquid = 'Liquid'
    Vapor = 'Vapor'



"""
Module for representing a component composition of a fluid mixture.

This class is intended to serialize/deserialize data in a format compatible with the VLXE API.
"""
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CalculationComposition")


@attr.s(auto_attribs=True)
class CalculationComposition:
    """Holds composition information for a component.
    
    This information to be stored in the `components: List[CalculationComposition]` attribute of input classes such as `FlashFixedTemperaturePressureCalculationInput` or `EosPropertiesTPnCalculationInput`.
    
    Attributes
    ----------
    component_id : str
        Id of the component.
    component_name : str
        Name of the component.
    amount : float
        Amount of the component (in units defined in the `units` attribute).
    """
    component_id: Union[Unset, str] = UNSET
    component_name: Union[Unset, None, str] = UNSET
    amount: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Dump `CalculationComposition` instance to a dict."""
        component_id = self.component_id
        component_name = self.component_name
        amount = self.amount

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if not isinstance(component_id, Unset):
            field_dict["componentId"] = component_id
        if not isinstance(component_name, Unset):
            field_dict["componentName"] = component_name
        if not isinstance(amount, Unset):
            field_dict["amount"] = amount

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Load `CalculationComposition` instance from a dict."""
        d = src_dict.copy()
        component_id = d.pop("componentId", UNSET)

        component_name = d.pop("componentName", UNSET)

        amount = d.pop("amount", UNSET)

        calculation_composition = cls(
            component_id=component_id,
            component_name=component_name,
            amount=amount,
        )

        return calculation_composition

    def __str__(self) -> str:
        """
        Returns a string representation of the `CalculationComposition` instance.
        
        Includes:
        - Component name, ID, and amount.
        """
        parts = []
        name = self.component_name if not isinstance(self.component_name, Unset) else "N/A"
        comp_id = self.component_id if not isinstance(self.component_id, Unset) else "N/A"
        amount = self.amount if not isinstance(self.amount, Unset) else "N/A"
        
        parts.append(f"Component: {name} (ID: {comp_id})")
        parts.append(f"  Amount: {amount}")

        return "\n".join(parts)



""" 
Class stores standard component metadata, including PC-SAFT parameters and thermodynamic constants.
"""
from typing import Any, Dict, Type, TypeVar, Union
import attr
from ..types import UNSET, Unset

T = TypeVar("T", bound="ApiFluidStandardComponent")

# Mapping from attribute name to VLXE API key
ATTRIBUTE_MAPPING: Dict[str, str] = {
    # general component data
    "formula": "formula",
    "dippr_database_id": "dipprDatabaseId",
    "name": "name",
    "is_alkane": "isAlkane",
    "sorting_order": "sortingOrder",
    "molar_mass": "molarMass",
    "critical_temperature": "criticalTemperature",
    "critical_pressure": "criticalPressure",
    "critical_volume": "criticalVolume",
    "acentric_factor": "acentricFactor",
    # model parameter data
    "volume_shift": "volumeShift",
    "pc_saft_epsilon": "pcSaftEpsilon",
    "pc_saft_sigma_0": "pcSaftSigma0",
    "pc_saft_sigma_1": "pcSaftSigma1",
    "pc_saft_sigma_2": "pcSaftSigma2",
    "pc_saft_sigma_3": "pcSaftSigma3",
    "pc_saft_sigma_4": "pcSaftSigma4",
    "pc_saftdm": "pcSaftdm",
    "pc_saft_ab_active": "pcSaftAbActive",
    "pc_saft_ab_kappa": "pcSaftAbKappa",
    "pc_saft_ab_epsilon": "pcSaftAbEpsilon",
    "pc_saft_ab_scheme": "pcSaftAbScheme",
    "pc_saft_polar_active": "pcSaftPolarActive",
    "pc_saft_polarx": "pcSaftPolarx",
    "pc_saft_polar_d": "pcSaftPolarD",
    # heat capacity coefficients
    "cp_ig_poly_c0": "CpIgPolyC0",
    "cp_ig_poly_c1": "CpIgPolyC1",
    "cp_ig_poly_c2": "CpIgPolyC2",
    "cp_ig_poly_c3": "CpIgPolyC3",
    "cp_ig_poly_c4": "CpIgPolyC4",
    "cp_ig_poly_c5": "CpIgPolyC5",
    "cp_ig_dippr_c0": "CpIgDipprC0",
    "cp_ig_dippr_c1": "CpIgDipprC1",
    "cp_ig_dippr_c2": "CpIgDipprC2",
    "cp_ig_dippr_c3": "CpIgDipprC3",
    "cp_ig_dippr_c4": "CpIgDipprC4",
    "cp_ig_dippr_c5": "CpIgDipprC5",
    "cp_ig_dippr_c6": "CpIgDipprC6",
    # thermodynamic constants
    "ideal_gas_gibbs_energy_of_formation": "IdealGasGibbsEnergyOfFormation",
    "ideal_gas_enthalpy_of_formation": "IdealGasEnthalpyOfFormation",
}
    
@attr.s(auto_attribs=True)
class ApiFluidStandardComponent:
    """Information for a standard fluid component.

    Stores the parameters for a *standard* component, which is to be stored in the `ApiFluid.standards` attribute.
    
    Attributes
    ----------
    formula : str
        Formula of the standard component.
    dippr_database_id : int
        Dippr database id of the standard component.
    name : str
        Name of the standard component.
    is_alkane : bool
        Whether the standard component is an alkane.
    sorting_order : int
        Index for sorting the standard components.
    molar_mass : float
        Molar mass (g/mol) of the standard component.
    critical_temperature : float
        Critical temperature (K) of the standard component.
    critical_pressure : float
        Critical pressure (bar) of the standard component.
    critical_volume : float
        Critical volume (cm^3/mol) of the standard component.
    acentric_factor : float
        Acentric factor (-) of the standard component.
    volume_shift : float
        Volume shift (cm3/g) of the standard component.
    pc_saft_epsilon : float
        PC-SAFT epsilon (K) of the standard component.
    pc_saft_sigma_0 : float
        PC-SAFT sigma_0 (Angstrom) of the standard component.
    pc_saft_sigma_1 : float
        PC-SAFT sigma_1 (Angstrom) of the standard component.
    pc_saft_sigma_2 : float
        PC-SAFT sigma_2 (1/K) of the standard component.
    pc_saft_sigma_3 : float
        PC-SAFT sigma_3 (Angstrom) of the standard component.
    pc_saft_sigma_4 : float
        PC-SAFT sigma_4 (1/K) of the standard component.
    pc_saftdm : float
        PC-SAFT dm (mol/g) of the standard component.
    pc_saft_ab_active : bool
        PC-SAFT association active of the standard component.
    pc_saft_ab_kappa : float
        PC-SAFT association bond volume (-) of the standard component.
    pc_saft_ab_epsilon : float
        PC-SAFT association energy (K) of the standard component.
    pc_saft_ab_scheme : str
        PC-SAFT association scheme (-) of the standard component.
    pc_saft_polar_active : bool
        PC-SAFT polar active of the standard component.
    pc_saft_polarx : float
        PC-SAFT polar x (-) of the standard component.
    pc_saft_polar_d : float
        PC-SAFT polar dipole strength (D) of the standard component.
    cp_ig_poly_c[i] : float
        Ideal gas heat capacity polynomial coefficients of the standard component.
    cp_ig_dippr_c[i] : float
        Ideal gas heat capacity DIPPR coefficients of the standard component.
    ideal_gas_gibbs_energy_of_formation : float
        Ideal gas Gibbs energy of formation (kJ/kg) of the standard component.
    ideal_gas_enthalpy_of_formation : float
        Ideal gas Enthalpy of formation (kJ/kg) of the standard component.
    """
    # General component data
    formula: Union[Unset, None, str] = UNSET
    dippr_database_id: Union[Unset, int] = UNSET
    name: Union[Unset, None, str] = UNSET
    is_alkane: Union[Unset, bool] = UNSET
    sorting_order: Union[Unset, int] = UNSET
    molar_mass: Union[Unset, float] = UNSET
    critical_temperature: Union[Unset, float] = UNSET
    critical_pressure: Union[Unset, float] = UNSET
    critical_volume: Union[Unset, float] = UNSET
    acentric_factor: Union[Unset, float] = UNSET
    # Model parameter data
    volume_shift: Union[Unset, float] = UNSET
    pc_saft_epsilon: Union[Unset, float] = UNSET
    pc_saft_sigma_0: Union[Unset, float] = UNSET
    pc_saft_sigma_1: Union[Unset, float] = UNSET
    pc_saft_sigma_2: Union[Unset, float] = UNSET
    pc_saft_sigma_3: Union[Unset, float] = UNSET
    pc_saft_sigma_4: Union[Unset, float] = UNSET
    pc_saftdm: Union[Unset, float] = UNSET
    pc_saft_ab_active: Union[Unset, bool] = UNSET
    pc_saft_ab_kappa: Union[Unset, float] = UNSET
    pc_saft_ab_epsilon: Union[Unset, float] = UNSET
    pc_saft_ab_scheme: Union[Unset, str] = UNSET
    pc_saft_polar_active: Union[Unset, bool] = UNSET
    pc_saft_polarx: Union[Unset, float] = UNSET
    pc_saft_polar_d: Union[Unset, float] = UNSET
    # Heat capacity coefficients
    cp_ig_poly_c0: Union[Unset, float] = UNSET
    cp_ig_poly_c1: Union[Unset, float] = UNSET
    cp_ig_poly_c2: Union[Unset, float] = UNSET
    cp_ig_poly_c3: Union[Unset, float] = UNSET
    cp_ig_poly_c4: Union[Unset, float] = UNSET
    cp_ig_poly_c5: Union[Unset, float] = UNSET
    cp_ig_poly_c6: Union[Unset, float] = UNSET
    cp_ig_poly_c7: Union[Unset, float] = UNSET
    cp_ig_dippr_c0: Union[Unset, float] = UNSET
    cp_ig_dippr_c1: Union[Unset, float] = UNSET
    cp_ig_dippr_c2: Union[Unset, float] = UNSET
    cp_ig_dippr_c3: Union[Unset, float] = UNSET
    cp_ig_dippr_c4: Union[Unset, float] = UNSET
    cp_ig_dippr_c5: Union[Unset, float] = UNSET
    cp_ig_dippr_c6: Union[Unset, float] = UNSET
    # Thermodynamic constants
    ideal_gas_gibbs_energy_of_formation: Union[Unset, float] = UNSET
    ideal_gas_enthalpy_of_formation: Union[Unset, float] = UNSET
    extra_attrs : Union[Unset, Dict[str, Any]] = UNSET   # Extra attributes not explicitly defined in the class

    def to_dict(self) -> Dict[str, Any]:
        """Dump `ApiFluidStandardComponent` instance to a dict."""
        result: Dict[str, Any] = {}
        for attr_name, json_key in ATTRIBUTE_MAPPING.items():
            val = getattr(self, attr_name)
            if not isinstance(val, Unset):
                result[json_key] = val
        return result

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Load `ApiFluidStandardComponent` instance from a dict."""
        reverse_map = {v: k for k, v in ATTRIBUTE_MAPPING.items()}
        kwargs = {
            attr_name: src_dict.get(json_key, UNSET)
            for json_key, attr_name in reverse_map.items()
        }
        return cls(**kwargs)
    
    def __str__(self) -> str:
        """
        Returns a compact string representation of the `ApiFluidStandardComponent` instance.
        
        Includes only attributes that are not `Unset`.
        """
        items = []
        for attr_name, json_key in ATTRIBUTE_MAPPING.items():
            val = getattr(self, attr_name)
            if not isinstance(val, Unset):
                items.append(f"{attr_name}={val}")
        return ", ".join(items)



""" 
Class stores block component metadata, including PC-SAFT parameters and thermodynamic constants.
"""
from typing import Any, Dict, Type, TypeVar, Union
import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ApiFluidComponentBlock")

# Mapping from attribute name to VLXE API key
ATTRIBUTE_MAPPING: Dict[str, str] = {
    # general component data
    "sorting_order": "sortingOrder",
    "block_massfraction": "blockMassfraction",
    "block_name": "blockName",
    "monomer_name": "monomerName",
    "monomer_molar_mass": "monomerMolarMass",
    # model parameter data
    "pc_saft_epsilon": "pcSaftEpsilon",
    "pc_saft_sigma_0": "pcSaftSigma0",
    "pc_saft_sigma_1": "pcSaftSigma1",
    "pc_saft_sigma_2": "pcSaftSigma2",
    "pc_saft_sigma_3": "pcSaftSigma3",
    "pc_saft_sigma_4": "pcSaftSigma4",
    "pc_saftdm": "pcSaftdm",
    "pc_saft_ab_active": "pcSaftAbActive",
    "pc_saft_ab_kappa": "pcSaftAbKappa",
    "pc_saft_ab_epsilon": "pcSaftAbEpsilon",
    "pc_saft_ab_scheme": "pcSaftAbScheme",
    "pc_saft_polar_active": "pcSaftPolarActive",
    "pc_saft_polarx": "pcSaftPolarx",
    "pc_saft_polar_d": "pcSaftPolarD",
    "sle_c": "sleC",
    "sle_hu": "sleHu",
    "sle_density_amorphous": "sleDensityAmorphous",
    "sle_density_crystalline": "sleDensityCrystalline",
    "sle_tss": "sleTss",
    "sle_hss": "sleHss",
    # heat capacity coefficients
    "cp_ig_poly_c0": "CpIgPolyC0",
    "cp_ig_poly_c1": "CpIgPolyC1",
    "cp_ig_poly_c2": "CpIgPolyC2",
    "cp_ig_poly_c3": "CpIgPolyC3",
    "cp_ig_poly_c4": "CpIgPolyC4",
    "cp_ig_poly_c5": "CpIgPolyC5",
    "cp_ig_poly_c6": "CpIgPolyC6",
    "cp_ig_poly_c7": "CpIgPolyC7",
    "cp_ig_dippr_c0": "CpIgDipprC0",
    "cp_ig_dippr_c1": "CpIgDipprC1",
    "cp_ig_dippr_c2": "CpIgDipprC2",
    "cp_ig_dippr_c3": "CpIgDipprC3",
    "cp_ig_dippr_c4": "CpIgDipprC4",
    "cp_ig_dippr_c5": "CpIgDipprC5",
    "cp_ig_dippr_c6": "CpIgDipprC6",
    # thermodynamic constants
    "ideal_gas_gibbs_energy_of_formation": "IdealGasGibbsEnergyOfFormation",
    "ideal_gas_enthalpy_of_formation": "IdealGasEnthalpyOfFormation",
}


@attr.s(auto_attribs=True)
class ApiFluidComponentBlock:
    """
    Values for a component block.

    Attributes
    ----------
    sorting_order : int
        Sorting order for the component block.
    block_massfraction : float
        Mass fraction of the component block.
    block_name : str
        Name of the component block.
    monomer_name : str
        Name of the monomer.
    monomer_molar_mass : float
        Molar mass of the monomer.
    pc_saft_epsilon : float
        Segment interaction energy (K) of the component block.
    pc_saft_sigma_i : float
        Segment diameter (A) of the component block.
    pc_saftdm : float
        Ratio of segment number to molar mass for the component block.
    pc_saft_ab_active : bool
        Association active for the component block.
    pc_saft_ab_kappa : float
        Association volume (-) of the component block.
    pc_saft_ab_epsilon : float
        Association energy (K) of the component block.
    pc_saft_ab_scheme : str
        Association scheme of the component block.
    pc_saft_polar_active : bool
        Polar active for the component block.
    pc_saft_polarx : float
        Polar x parameter (-) of the component block.
    pc_saft_polar_d : float
        Dipolar moment (Debye) of the component block.
    sle_c : float
        SLE c parameter (-) of the component block.
    sle_hu : float
        SLE Hu parameter (-) of the component block.
    sle_density_amorphous : float
        SLE density amorphous (g/cm^3) of the component block.
    sle_density_crystalline : float
        SLE density crystalline (g/cm^3) of the component block.
    sle_tss : float
        SLE tss parameter (-) of the component block.
    sle_hss : float
        SLE hss parameter (-) of the component block.
    cp_ig_poly_c[i] : float
        Ideal gas heat capacity polynomial coefficients of the component block. 
    cp_ig_dippr_c[i] : float
        Ideal gas heat capacity dippr coefficients of the component block.
    ideal_gas_gibbs_energy_of_formation : float
        Ideal gas gibbs energy of formation (kJ/kg) of the component block.
    ideal_gas_enthalpy_of_formation : float
        Ideal gas enthalpy of formation (kJ/kg) of the component block.
    """
    # General component data
    sorting_order:    Union[Unset, int]   = UNSET
    block_massfraction: float = UNSET
    block_name:      Union[Unset, None, str] = UNSET
    monomer_name:    Union[Unset, None, str] = UNSET
    monomer_molar_mass: float = UNSET
    # Model parameter data
    pc_saft_epsilon: float = UNSET
    pc_saft_sigma_0: float = UNSET
    pc_saft_sigma_1: float = UNSET
    pc_saft_sigma_2: float = UNSET
    pc_saft_sigma_3: float = UNSET
    pc_saft_sigma_4: float = UNSET
    pc_saftdm:      float = UNSET
    pc_saft_ab_active: bool = UNSET
    pc_saft_ab_kappa: float = UNSET
    pc_saft_ab_epsilon: float = UNSET
    pc_saft_ab_scheme: str   = UNSET
    pc_saft_polar_active: bool  = UNSET
    pc_saft_polarx: float = UNSET
    pc_saft_polar_d: float = UNSET
    sle_c:            float = UNSET
    sle_hu:           float = UNSET
    sle_density_amorphous:  float = UNSET
    sle_density_crystalline: float = UNSET
    sle_tss:           float = UNSET
    sle_hss:           float = UNSET
    # Heat capacity coefficients
    cp_ig_poly_c0: float = UNSET
    cp_ig_poly_c1: float = UNSET
    cp_ig_poly_c2: float = UNSET
    cp_ig_poly_c3: float = UNSET
    cp_ig_poly_c4: float = UNSET
    cp_ig_poly_c5: float = UNSET
    cp_ig_poly_c6: float = UNSET
    cp_ig_poly_c7: float = UNSET
    cp_ig_dippr_c0: float = UNSET
    cp_ig_dippr_c1: float = UNSET
    cp_ig_dippr_c2: float = UNSET
    cp_ig_dippr_c3: float = UNSET
    cp_ig_dippr_c4: float = UNSET
    cp_ig_dippr_c5: float = UNSET
    cp_ig_dippr_c6: float = UNSET
    # Thermodynamic constants
    ideal_gas_gibbs_energy_of_formation: float = UNSET
    ideal_gas_enthalpy_of_formation:    float = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Dump `ApiFluidComponentBlock` instance to a dict."""
        result: Dict[str, Any] = {}
        for attr_name, json_key in ATTRIBUTE_MAPPING.items():
            value = getattr(self, attr_name)
            if value is not UNSET:
                result[json_key] = value
        return result

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Load `ApiFluidComponentBlock` instance from a dict."""
        d = src_dict.copy()
        init_kwargs: Dict[str, Any] = {}
        for attr_name, json_key in ATTRIBUTE_MAPPING.items():
            init_kwargs[attr_name] = d.pop(json_key, UNSET)
        return cls(**init_kwargs)

    def __str__(self) -> str:
        """
        Returns a compact string representation of the `ApiFluidComponentBlock` instance.
        
        Includes only attributes that are not `Unset`.
        """
        items = []
        for attr_name, json_key in ATTRIBUTE_MAPPING.items():
            val = getattr(self, attr_name)
            if not isinstance(val, Unset):
                items.append(f"{attr_name}={val}")
        return ", ".join(items)



"""
Module for representing a flash calculation input.
"""
from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.calculation_composition import CalculationComposition
from ..models.api_fluid import ApiFluid
from ..types import UNSET, Unset
from ..str_util import default_input_str

T = TypeVar("T", bound="FlashFixedTemperaturePressureCalculationInput")

@attr.s(auto_attribs=True)
class FlashFixedTemperaturePressureCalculationInput:
    """Input for flash calculation.

    Attributes
    ----------
    access_key : str
        Access key for the API.
    components : List[CalculationComposition]
        Component composition
    units : str 
        Units used for input and output.
    fluidid : str 
        Id of fluid on webserver. Must be defined if `fluid` argument is not given.
    fluid : ApiFluid
        Fluid information
    temperature : float
        Temperature in units given in `units` argument.
    pressure : float
        Pressure in units given in `units` argument.
    """
    access_key: str
    components: List[CalculationComposition]
    units: str
    fluidid: Union[Unset, str] = UNSET
    fluid: Union[Unset, ApiFluid] = UNSET
    temperature: Union[Unset, float] = UNSET
    pressure: Union[Unset, float] = UNSET
    __str__ = default_input_str
    
    def to_dict(self) -> Dict[str, Any]:
        """Dump `FlashFixedTemperaturePressureCalculationInput` instance to a dict."""
        access_key = self.access_key
        components = []
        for components_item_data in self.components:
            components_item = components_item_data.to_dict()
            components.append(components_item)

        units = self.units
        temperature = self.temperature
        pressure = self.pressure

        fluidid: Union[Unset, str] = UNSET
        if not isinstance(self.fluidid, Unset):
            fluid = self.fluidid

        fluid: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.fluid, Unset):
            fluid = self.fluid.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "accessKey": access_key,
                "components": components,
            }
        )
        if not isinstance(units, Unset):
            field_dict["units"] = units
        if not isinstance(fluidid, Unset):
            field_dict["fluidId"] = fluidid
        if not isinstance(fluid, Unset):
            field_dict["fluid"] = fluid
        if not isinstance(temperature, Unset):
            field_dict["temperature"] = temperature
        if not isinstance(pressure, Unset):
            field_dict["pressure"] = pressure

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Load `FlashFixedTemperaturePressureCalculationInput` instance from a dict."""
        d = src_dict.copy()
        access_key = d.pop("accessKey")
        units = d.pop("units", UNSET)

        components = []
        _components = d.pop("components")
        for components_item_data in _components:
            components_item = CalculationComposition.from_dict(
                components_item_data)

            components.append(components_item)

        _fluidid = d.pop("fluidId", UNSET)
        fluidid: Union[Unset, str]
        if isinstance(_fluidid, Unset):
            fluidid = UNSET
        else:
            fluidid = _fluidid

        _fluid = d.pop("fluid", UNSET)
        fluid: Union[Unset, ApiFluid]
        if isinstance(_fluid, Unset):
            fluid = UNSET
        else:
            fluid = ApiFluid.from_dict(_fluid)

        temperature = d.pop("temperature", UNSET)
        pressure = d.pop("pressure", UNSET)

        flash_calculation_input = cls(
            access_key=access_key,
            components=components,
            units=units,
            fluidid=fluidid,
            fluid=fluid,
            temperature=temperature,
            pressure=pressure,
        )

        return flash_calculation_input

    @classmethod
    def from_input_object(cls: Type[T], input_object: "FlashFixedTemperaturePressureCalculationInput") -> T:
        """Load `FlashFixedTemperaturePressureCalculationInput` instance from a similar instance, such as `FlashFixedTemperaturePressureCalculationInput`.
        This is useful for copying data from one instance to another.
        
        Parameters
        ----------
        input_object : Union[EosPropertiesTPnCalculationInput, FlashFixedTemperaturePressureCalculationInput, CloudPointFixedPressureCalculationInput]
            Instance to copy data from.
        
        Returns
        -------
        FlashFixedTemperaturePressureCalculationInput
            New instance with copied data.
        """
        d = input_object.to_dict()
        return cls.from_dict(d)



import pytest
from typing import Union
from equia.models import (
    ProblemDetails,
    FluidAddInput, FluidAddResult, 
    FluidGetInput, FluidGetResult, 
    FluidDeleteInput, FluidDeleteResult
)
from equia.equia_client import EquiaClient
from equia.demofluids.demofluid1_nHexane_Ethylene_HDPE7 import demofluid1_nHexane_Ethylene_HDPE7
from utility.api_access import ApiAccess

@pytest.mark.asyncio
async def test_call_fluid():
    # Add fluid
    clientAdd = EquiaClient(ApiAccess.url, ApiAccess.access_key)

    inputAdd: FluidAddInput = clientAdd.get_fluid_add_input()
    inputAdd.fluid = demofluid1_nHexane_Ethylene_HDPE7()

    resultAdd: Union[FluidAddResult, ProblemDetails] = await clientAdd.call_fluid_add_async(inputAdd)

    await clientAdd.cleanup()

    #assert result.status == 400
    print(resultAdd)
    if isinstance(resultAdd, ProblemDetails):
        print(resultAdd.exception_info)
        print(resultAdd.exception_info.message)
        print(resultAdd.exception_info.stack_trace)
    assert resultAdd.success is True

    # Get fluid    
    clientGet = EquiaClient(ApiAccess.url, ApiAccess.access_key)

    inputGet: Union[FluidGetInput, ProblemDetails] = clientGet.get_fluid_get_input()
    inputGet.fluidid = resultAdd.fluidid

    resultGet: Union[FluidGetResult, ProblemDetails] = await clientGet.call_fluid_get_async(inputGet)

    await clientGet.cleanup()

    #assert result.status == 400
    assert resultGet.success is True
    assert len(resultGet.fluid.standards) == 2
    assert len(resultGet.fluid.polymers) == 1

    # Delete fluid
    clientDelete = EquiaClient(ApiAccess.url, ApiAccess.access_key)

    inputDelete: Union[FluidDeleteInput, ProblemDetails] = clientDelete.get_fluid_delete_input()
    inputDelete.fluidid = resultAdd.fluidid

    resultDelete: Union[FluidDeleteResult, ProblemDetails] = await clientDelete.call_fluid_delete_async(inputDelete)

    await clientDelete.cleanup()

    #assert result.status == 400
    assert resultDelete.success is True



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ApiFluidKij")


@attr.s(auto_attribs=True)
class ApiFluidKij:
    """
    Set of Kij information. To be used in the `ApiFluid.kij` attribute.
    
    Attributes
    ----------
    index_i : int
        Index of component i.
    index_j : int
        Index of component j.
    kija : float
        `kija` value for the expression `kij = kija + kijb * T(K)`.
    kijb : float
        `kijb` value for the expression `kij = kija + kijb * T(K)`.
    """
    index_i: Union[Unset, int] = UNSET
    index_j: Union[Unset, int] = UNSET
    kija: Union[Unset, float] = UNSET
    kijb: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Dump `ApiFluidKij` instance to a dict."""
        index_i = self.index_i
        index_j = self.index_j
        kija = self.kija
        kijb = self.kijb

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if not isinstance(index_i, Unset):
            field_dict["indexI"] = index_i
        if not isinstance(index_j, Unset):
            field_dict["indexJ"] = index_j
        if not isinstance(kija, Unset):
            field_dict["kija"] = kija
        if not isinstance(kijb, Unset):
            field_dict["kijb"] = kijb

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Load `ApiFluidKij` instance from a dict."""
        d = src_dict.copy()
        index_i = d.pop("indexI", UNSET)

        index_j = d.pop("indexJ", UNSET)

        kija = d.pop("kija", UNSET)

        kijb = d.pop("kijb", UNSET)

        api_fluid_kij = cls(
            index_i=index_i,
            index_j=index_j,
            kija=kija,
            kijb=kijb,
        )

        return api_fluid_kij

    def __str__(self) -> str:
        """
        Returns a string representation in the form: Kij(i, j) = (kija, kijb)
        """
        i = self.index_i if not isinstance(self.index_i, Unset) else "?"
        j = self.index_j if not isinstance(self.index_j, Unset) else "?"
        a = self.kija if not isinstance(self.kija, Unset) else "?"
        b = self.kijb if not isinstance(self.kijb, Unset) else 0
        return f"Kij({i}, {j}) = ({a}, {b})"



import aiohttp
from typing import Any, Dict, Union

from equia.units import Units
from equia.models import StatusInput, StatusResult, CloudPointFixedTemperatureCalculationInput, CloudPointFixedPressureCalculationInput, CloudPointCalculationResult, \
    FlashFixedTemperaturePressureCalculationInput, FlashCalculationResult, \
    BatchFlashFixedTemperaturePressureCalculationInput, BatchFlashCalculationResult, \
    PhasediagramFixedTemperaturePressureCalculationInput, PhasediagramFixedTemperaturePressureCalculationResult, \
    FluidAddInput, FluidAddResult, FluidGetInput, FluidGetResult, FluidDeleteInput, FluidDeleteResult, \
    SlePointFixedPressureCalculationInput, SlePointFixedTemperaturePressureCalculationInput, SlePointCalculationResult, \
    EosPropertiesTPnCalculationInput, EosPropertiesTPnCalculationResult, \
    ProblemDetails \

class EquiaClient:
    """Class for making the calling to Equia API easy.
    
    Attributes
    ----------
    __base_url : str
        Base URL for the API. Includes the server address and the calculation endpoint.
    __access_key : str
        Access key for the API. This is used to authenticate the user.
    __session : aiohttp.ClientSession
        Session for making the requests to the API. This is used to manage the connection and the requests.
    """

    def __init__(self, base_url: str, access_key: str):
        self.__base_url = "{}/publicapi/".format(base_url)
        self.__access_key = access_key
        self.__session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=False))

    @classmethod
    def from_api_access(cls, api_access) -> "EquiaClient":
        """Create an `EquiaClient` instance from an `ApiAccess` object."""
        return cls(api_access.url, api_access.access_key)
    
    async def cleanup(self):
        """Close session"""
        await self.__session.close()

    def __append_body(self, body: Dict[str, Any]) -> Dict[str, Any]:
        """Method that appends access_key to the body of the request"""
        user = {
            "accesskey": self.__access_key,
        }
        return {**body, **user}

    async def __post_async(self, endpoint: str, json_body: Dict[str, Any], from_dict) -> Union[ProblemDetails, Any]:
        """Prepare the URL and the body and makes the POST request to the API"""
        url = "{}{endpoint}".format(self.__base_url, endpoint=endpoint)
        json_body = self.__append_body(json_body)
        try:
            async with self.__session.post(url, json=json_body) as resp:
                response = await resp.json()
                if resp.status == 200:
                    return from_dict(response)
                else:
                    return ProblemDetails.from_dict(response)
        except aiohttp.ClientConnectorError as e:
            print('Connection Error', str(e))
            return ProblemDetails.from_dict(e)

    async def call_status_get_async(
        self,
        body: StatusInput,
    ) -> Union[ProblemDetails, StatusResult]: 
        """Return status of the web server."""
        return await self.__post_async("Status/GetStatus", body.to_dict(), StatusResult.from_dict)

    async def call_fluid_get_async(
        self,
        body: FluidGetInput,
    ) -> Union[ProblemDetails, FluidGetResult]: 
        """Return fluid with given id."""  
        return await self.__post_async("Fluids/GetFluid", body.to_dict(), FluidGetResult.from_dict)

    async def call_fluid_add_async(
        self,
        body: FluidAddInput,
    ) -> Union[ProblemDetails, FluidAddResult]: 
        """Add given fluid to web server and return fluid id."""  
        return await self.__post_async("Fluids/AddFluid", body.to_dict(), FluidAddResult.from_dict)

    async def call_fluid_delete_async(
        self,
        body: FluidDeleteInput,
    ) -> Union[ProblemDetails, FluidDeleteResult]: 
        """Delete given fluid on the web server."""  
        return await self.__post_async("Fluids/DeleteFluid", body.to_dict(), FluidDeleteResult.from_dict)

    # region: Flash calculations
    async def call_flash_fixed_temperature_pressure_async(
        self,
        body: FlashFixedTemperaturePressureCalculationInput,
    ) -> Union[ProblemDetails, FlashCalculationResult]:
        """Perform flash calculation."""
        return await self.__post_async(f"Calculations/Flash/fixedtemperaturepressure", body.to_dict(), FlashCalculationResult.from_dict)

    async def call_batchflash_fixed_temperature_pressure_async(
        self,
        body: BatchFlashFixedTemperaturePressureCalculationInput,
    ) -> Union[ProblemDetails, BatchFlashCalculationResult]:
        """Perform batch flash calculation."""
        return await self.__post_async(f"Calculations/BatchFlash/fixedtemperaturepressure", body.to_dict(), BatchFlashCalculationResult.from_dict)

    async def call_cloud_point_fixed_pressure_async(
        self,
        body: CloudPointFixedPressureCalculationInput,
    ) -> Union[ProblemDetails, CloudPointCalculationResult]:
        """Perform cloud point calculation."""
        return await self.__post_async(f"Calculations/CloudPoint/fixedpressure", body.to_dict(), CloudPointCalculationResult.from_dict)

    async def call_cloud_point_fixed_temperature_async(
        self,
        body: CloudPointFixedTemperatureCalculationInput,
    ) -> Union[ProblemDetails, CloudPointCalculationResult]:
        """Perform cloud point calculation."""
        return await self.__post_async(f"Calculations/CloudPoint/fixedtemperature", body.to_dict(), CloudPointCalculationResult.from_dict)

    async def call_sle_point_fixed_pressure_async(
        self,
        body: SlePointFixedPressureCalculationInput,
    ) -> Union[ProblemDetails, SlePointCalculationResult]:
        """Perform SLE point calculation."""
        return await self.__post_async(f"Calculations/Sle/Point/fixedpressure", body.to_dict(), SlePointCalculationResult.from_dict)

    async def call_sle_point_fixed_temperature_pressure_async(
        self,
        body: SlePointFixedTemperaturePressureCalculationInput,
    ) -> Union[ProblemDetails, SlePointCalculationResult]:
        """Perform SLE point calculation."""
        return await self.__post_async(f"Calculations/Sle/Point/fixedtemperaturepressure", body.to_dict(), SlePointCalculationResult.from_dict)

    async def call_phasediagram_standard_async(
        self,
        body: PhasediagramFixedTemperaturePressureCalculationInput,
    ) -> Union[ProblemDetails, PhasediagramFixedTemperaturePressureCalculationResult]:
        """Perform Phasediagram standard calculation."""
        return await self.__post_async(f"Calculations/Phasediagram/FixedComposition", body.to_dict(), PhasediagramFixedTemperaturePressureCalculationResult.from_dict)

    async def call_eospropertiestpn_async(
        self,
        body: EosPropertiesTPnCalculationInput,
    ) -> Union[ProblemDetails, EosPropertiesTPnCalculationResult]:
        """Perform Eos property TPn calculation."""
        return await self.__post_async("Calculations/Properties/EosTPn", body.to_dict(), EosPropertiesTPnCalculationResult.from_dict)
    # endregion
    
    # region: Input getters
    def get_status_input(self) -> StatusInput:
        """Returns status argument filled with standard input."""
        return StatusInput(access_key=self.__access_key)

    def get_flash_fixed_temperature_pressure_input(self) -> FlashFixedTemperaturePressureCalculationInput:
        """Returns flash argument filled with standard input."""
        return FlashFixedTemperaturePressureCalculationInput(access_key=self.__access_key, units=self.__get_units(), components=[], fluidid="")

    def get_batchflash_fixed_temperature_pressure_input(self) -> BatchFlashFixedTemperaturePressureCalculationInput:
        """Returns batch flash argument filled with standard input."""
        return BatchFlashFixedTemperaturePressureCalculationInput(access_key=self.__access_key, units=self.__get_units(), points=[], fluidid="")

    def get_cloud_point_fixed_pressure_input(self) -> CloudPointFixedPressureCalculationInput:
        """Returns cloud point argument filled with standard input."""
        return CloudPointFixedPressureCalculationInput(access_key=self.__access_key, units=self.__get_units(), components=[], fluidid="")

    def get_cloud_point_fixed_temperature_input(self) -> CloudPointFixedTemperatureCalculationInput:
        """Returns cloud point argument filled with standard input."""
        return CloudPointFixedTemperatureCalculationInput(access_key=self.__access_key, units=self.__get_units(), components=[], fluidid="")

    def get_eospropertiestpn_input(self) -> EosPropertiesTPnCalculationInput:
        """Returns Eos property TPN argument filled with standard input."""
        return EosPropertiesTPnCalculationInput(access_key=self.__access_key, units=self.__get_units(), components=[], fluidid="", volumetype="Auto")

    def get_sle_point_fixed_pressure_input(self) -> SlePointFixedPressureCalculationInput:
        """Returns SLE point argument filled with standard input."""
        return SlePointFixedPressureCalculationInput(access_key=self.__access_key, units=self.__get_units(), components=[], fluidid="")

    def get_sle_point_fixed_temperature_pressure_input(self) -> SlePointFixedTemperaturePressureCalculationInput:
        """Returns SLE point argument filled with standard input."""
        return SlePointFixedTemperaturePressureCalculationInput(access_key=self.__access_key, units=self.__get_units(), components=[], fluidid="")

    def get_phasediagram_standard_input(self) -> PhasediagramFixedTemperaturePressureCalculationInput:
        """Returns phase diagram standard argument filled with standard input."""
        return PhasediagramFixedTemperaturePressureCalculationInput(access_key=self.__access_key, units=self.__get_units(), components=[], fluidid="")

    def get_fluid_get_input(self) -> FluidGetInput:
        """Returns request fluid argument filled with standard input."""
        return FluidGetInput(access_key=self.__access_key)

    def get_fluid_add_input(self) -> FluidAddInput:
        """Returns new fluid argument filled with standard input."""
        return FluidAddInput(access_key=self.__access_key)

    def get_fluid_delete_input(self) -> FluidDeleteInput:
        """Returns new fluid argument filled with standard input."""
        return FluidDeleteInput(access_key=self.__access_key)
    # endregion
    
    # region: Unit getter
    def __get_units(self) -> str:
        """Returns units string filled with standard units."""
        units = Units()
        return str(units)



""" `models.str_util`

Utility functions for input/output string formatting for `*Input` and `*Result` objects.
"""
import attr
from .types import UNSET, Unset

def default_input_str(self) -> str:
    """Default string representation for `*Input` objects."""
    lines = ["", f"{self.__class__.__name__}:"]
    for fld in attr.fields(self.__class__):
        val = getattr(self, fld.name)
        out = val if not isinstance(val, (Unset, type(None))) else "N/A"
        # indent multi-line repr
        text = str(out).replace("\n", "\n    ")
        lines.append(f"  {fld.name}: {text}")
    return "\n".join(lines)

def default_result_str(self) -> str:
    """Default string representation for `*Result` objects."""
    parts = ["", f"{self.__class__.__name__}:"]
    for field in getattr(self, "__attrs_attrs__", []):
        name = field.name
        val = getattr(self, name)
        if isinstance(val, (Unset, type(None))):
            parts.append(f"  {name}: N/A")
        else:
            text = str(val).replace("\n", "\n    ")
            parts.append(f"  {name}: {text}")
    return "\n".join(parts)
